import java.io.*;
import org.xml.sax.*;
import org.xml.sax.Attributes;
import org.xml.sax.XMLReader;
import javax.xml.parsers.*;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.ContentHandler;
import org.xml.sax.ext.*;
import org.xml.sax.helpers.*;
import java.awt.image.*;
import com.sun.image.codec.jpeg.*;
import javax.swing.*;
import java.awt.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;
import javax.swing.border.*;
import java.util.Locale;
import java.util.ResourceBundle;
import java.text.MessageFormat;
import java.util.*;

/**
 * This class will parse an xml file and will display the content of it in a JTree.
 * If user select an image filename than it will show the image in the label next to the JTree.
 * @author Veronica Liesaputra
 */
public class XMLParser extends DefaultHandler
    implements ContentHandler,TreeSelectionListener
{
    private String dirname = "xmlfeedback/";
    
    /**
     * This is the tree that will be used to hold the content of the xml file.
     */
    private JTree tree;
    /**
     * This is the root node for tree.
     */
    private DefaultMutableTreeNode rootNode;
    /**
     * This hold the parent node of the current node that is currently just being added to tree.
     */
    private DefaultMutableTreeNode parentNode;
    /**
     * This hold the current node that is currently just being added to tree.
     */
    private DefaultMutableTreeNode currNode;
    /**
     * This is the tree model for tree.
     */
    private DefaultTreeModel treeModel;
    /**
     * This is the panel that hold the content pane of the frame-window.
     * It contains the tree and the button to close the window.
     */
    private static JPanel pane;
    /**
     * This is the scroll pane for displaying tree, so it is scrollable.
     */
    private JScrollPane scroll;
    /**
     * This is the scroll pane for displaying the image, so it is scrollable.
     */
    private JScrollPane picscroll;
    /**
     * This is the flag variable used to mark whether or not we currently reading
     * the string representation of an image.
     */
    private boolean readingImage = false;
    /**
     * This is the flag variable used to mark whether or not we currently reading the
     * image's width.
     */
    private boolean readingWidth = false;
    /**
     * This is the flag variable used to mark whether or not we currently reading the
     * image's height.
     */
    private boolean readingHeight = false;
    /**
     * This is the image file name.
     */
    private String imgFile = "";  
    /**
     * This is the string representation of the image read so far from the xml file.
     */
    private String cdataString = "";
    /**
     * This is a flag we use to say that we are reading a CDATA section at the moment.
     */
    private boolean readingCDATASection = false;
    /**
     * This is the frame to display the view of a xml file.
     */
    private JFrame frames;
    /**
     * This is the label to display image inside the xml file.
     */
    private JLabel lblIcon;
    /**
     * This is the split pane used to split between the tree and the image.
     */
    private JSplitPane splitPane;
    /**
     * This is the width of the image.
     */
    private int width;
    /**
     * This is the height of the image.
     */
    private int height;
    /**
     * This variable will hold the resource of the words that is stored in Messages.properties file.
     * The calling using messages.getString(someString) will caused someString to be translated
     * into some other string that is hold in that file.usually it will caused it to be translated
     * to the language that the user use or choose to have as stated in Locale.
     */
    private static ResourceBundle messages;
    /**
     * This is the array that hold all the images.
     */
    private ArrayList array = null;

    /**
     * This constructor will make a frame_window.
     * @param frame is the frame to display the view of the xml file.
     * @param msg hold the resource of the words that is stored in Messages.properties file.
     */
    public XMLParser(JFrame frame,ResourceBundle msg)
    {
	super();
	
	lblIcon = new JLabel(" ");
	lblIcon.setMinimumSize(new Dimension(100,100));
	lblIcon.setBackground(new Color(224,240,224));
	picscroll = new JScrollPane(lblIcon); 
	messages = msg;
	frames = frame;
    }

    /**
     * This method will setting up the content of array into the specified array list.
     * @param movie the array list to set content of array.
     */
    public void setArray (ArrayList movie)
    {
	array = movie;
    }

    /**
     * This method will setting up the xml parser and reader to allow us to get the
     * content inside the xml file.
     * @return the parser/reader needed to read the xml file.
     */
    final public XMLReader makeXMLReader()
    {
	XMLReader parser = null;
	try
	    {
		SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
		SAXParser saxParser = saxParserFactory.newSAXParser();
		parser = saxParser.getXMLReader();
		return parser;
	    }
	catch (ParserConfigurationException exp) {System.out.println("parser exp");}
	catch (SAXException exp2) {System.out.println("sax exp");}
	return parser;
    }

    /**
     * This method will tell whether or not we still reading a string which inside the
     * CDATA section.
     * @return whether or not we pass the CDATA section.
     */
    public boolean passedStartCDATASection()
    {
        return readingCDATASection;
    }

    /**
     * This method will get the startElement tag from the xml file. Start from here the JTree is build.
     * It means we create a new node in the tree.
     * namespace and localname are required only if namespace property is true.
     * @param namespace  its NameSpace URI.
     * @param localname the local name
     * @param type is the name of tag for this element.
     * @param att is the attributes attached to the element.
     */
    final public void startElement (final String namespace,final String localname,final String type,final Attributes att)
	throws SAXException
    {
	if (type == "Image")
	    readingImage = true;
	
	if (type == "Width")
	    readingWidth = true;

	if (type == "Height")
	    readingHeight = true;

	if (type == "View")
	    readingCDATASection = true; 
	else
	    {
		if ((type == "HISTORY")||(type == "BUGS")||(type == "COMMANDS")||(type == "COMMENTS"))
		    {
			rootNode = new DefaultMutableTreeNode(messages.getString(type));
			treeModel = new DefaultTreeModel(rootNode);
			
			parentNode = rootNode;
			tree = new JTree(treeModel);
			tree.setBackground(new Color(224,240,224));
			tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
			
			tree.addTreeSelectionListener(this);
			tree.setEditable(false);
			tree.setShowsRootHandles(true);
		    }
		else
		    {
			currNode = new DefaultMutableTreeNode(messages.getString(type) + " : ");
			treeModel.insertNodeInto(currNode,parentNode,parentNode.getChildCount());
			parentNode = currNode;
		    }
	    }
    }

    /**
     * This method will get the endElement tag for the xml file.
     * It means that we finished adding the new node to the tree.
     * namespace and localname are required only if namespace property is true.
     * @param namespace  its NameSpace URI.
     * @param localname the local name
     * @param type is the name of tag for this element.
     */
    final public void endElement (final String namespace,final String localname,final String type)
	throws SAXException
    {
	if (readingCDATASection == true)
	    {
		byte[] img = Base64.decode(cdataString);
		showImage(img,imgFile);
		cdataString = "";
		imgFile = "";
		readingWidth = false;
		readingHeight = false;
		readingCDATASection = false;
		readingImage = false;
	    }
	else
	    {
		if ((type == "HISTORY")||(type == "BUGS")||(type == "COMMANDS")||(type == "COMMENTS"))
		    {
			Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
			
			scroll = new JScrollPane(tree);
			scroll.setPreferredSize(new Dimension(400,(dim.height - 100)));
			splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
						   scroll,picscroll);
			splitPane.setOneTouchExpandable(true);
			splitPane.setDividerLocation(400);

			Dimension minimumSize = new Dimension((dim.width - 500),(dim.height - 100));
			picscroll.setPreferredSize(minimumSize);
			
			pane = new JPanel();
			pane.setBackground(new Color(176,208,176));
			pane.add(splitPane);
			pane.setBorder(new EmptyBorder(new Insets(10,10,10,10)));
			Container comp = frames.getContentPane();
			comp.add(pane);
			frames.pack();
			frames.setVisible(true);
			return;
		    }
		else
		    {
			parentNode = (DefaultMutableTreeNode) parentNode.getParent();
		    }
	    }
    }
 
    /**
     * This will return the content pane of the frame.
     * @return the content pane of the frame.
     */
    public JPanel getPane()
    {
	return pane;
    }

    /**
     * This method will save the image save in the xml file to a jpeg file with the
     * specified filename.
     * It will make an image icon out of this byte[] and added to the array list of 
     * image icon.
     * @param img this is the byte[] representation of the image save in xml file.
     * @param filename the file name where the image should be saved.
     */
    public void showImage (byte[] img,String filename)
    {
	ImageIcon small_icon = new ImageIcon(img);
	ImageIcon icon = new ImageIcon(small_icon.getImage().getScaledInstance(width,height,Image.SCALE_SMOOTH));
	
	if (array != null)
	    array.add(0,icon);

	BufferedImage image = new BufferedImage(width,height, BufferedImage.TYPE_INT_RGB);
	Graphics2D big = image.createGraphics();
	
	big.drawImage(icon.getImage(),0,0,null);
	
	try 
	    {
		FileOutputStream out = new FileOutputStream(dirname + filename);
		
		JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
		JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(image);
		param.setQuality(1.0f, false);
		encoder.setJPEGEncodeParam(param);
		encoder.encode(image);
	    } 
	catch (Exception ex) {ex.printStackTrace();}

    }

    /**
     * This will append the content to the content's of the current node.
     * @param ch The characters from the XML document.
     * @param start The start position in the array.
     * @param length The number of characters to read from the array.
     */
    final public void characters (final char[] ch, final int start, final int len)
	throws SAXException
    {
	String text = new String(ch,start,len);
	String text1 = text.trim();
	if (text1.length() > 0)
	    {
		if(passedStartCDATASection() == true)
		    {
			cdataString += text1;
		    }
		else
		    {
			if (readingImage == true)
			    {
			       imgFile = text1;
			       readingImage = false;
			    }
			else if (readingWidth == true)
			    {
				width = Integer.parseInt(text1);
				readingWidth = false;
			    }
			else if (readingHeight == true)
			    {
				height = Integer.parseInt(text1);
				readingHeight = false;
			    }
			
			String txt = (String) currNode.getUserObject();
			currNode.setUserObject(txt + text1);
		    }
	    }
    }

    /**
     * This method is a tree listener that listens to which node user selected at the moment.
     * If user selected node that contains the image filename, then it will display the image
     * to the label next to the JTree.
     * @param e the tree selection event.
     */
    public void valueChanged (TreeSelectionEvent e)
    {
	DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();

	if (node == null)
	    return;
	
	 if (node.isLeaf() == true)
	     {
		 String content = (String) node.getUserObject();
		 StringTokenizer st = new StringTokenizer(content,": ");
		 String imgName;

		 if (st.hasMoreTokens())
		     {
			 if (st.nextToken().equals(messages.getString("Title")))
			     {
				 lblIcon.setIcon(new ImageIcon(dirname + st.nextToken()));
				 return;
			     }
			 else
			     return;
		     }
		 else
		     return;
	     }
	 else
	     return;
    }
} 












